#include "../../internal-include/BootLoader/boot_loader.hpp"
#include "../../internal-include/GPU/gpu_internal.hpp"
#include <PSX/System/IOPorts/dma_io.hpp>
#include <stdio.h>

extern "C" uint32_t __heap_start;
extern "C" uint32_t __bss_start;
extern "C" uint32_t __bss_end;
extern "C" uint32_t __bss_len;
extern "C" uint32_t __planschi_start;
extern "C" uint32_t __planschi_end;

namespace JabyEngine {
    namespace boot {
        namespace Start {
            static void enable_DMA() {
                DMA_IO::DPCR = DMA_IO::DPCR_t(DMA_IO::DPCR).set(DMA_IO::DPCR_t::SPUEnable).set(DMA_IO::DPCR_t::GPUEnable).set(DMA_IO::DPCR_t::CDROMEnable);
            }

            static void setup() {
                enable_DMA();

                SPU::stop_voices();
                
                CD::setup();
                Timer::setup();

                GPU::setup();
                GPU::display_logo();
                
                //Pause??
                SPU::setup();
            }
        }
    }

    void start() {
        printf("Starting Planschbecken\n");
        printf("Heap starts @0x%p\n", &__heap_start);
        printf("BSS      from 0x%p to 0x%p (%u)\n", &__bss_start, &__bss_end, __bss_len);
        printf("PLANSCHI from 0x%p to 0x%p\n", &__planschi_start, &__planschi_end);
        boot::Start::setup();
        
        printf("Running main...\n");
        GPU::internal::wait_vsync(0);
        run();
    }
}